@@ -24,6 +24,7 @@ import threading
 import argparse
 import queue
 import inotifyx
+import pathlib
 
 from .scanDirWatcher import ScanDirWatcher
 from .safeINotifier import SafeINotifier
@@ -54,6 +55,9 @@ class BeamtimeWatcher:
             self.__config = load_config(options.config) or {}
             get_logger().debug("CONFIGURATION: %s" % str(self.__config))
 
+        #: (:obj:`str`) home directory
+        self.__homepath = str(pathlib.Path.home())
+
         #: (:obj:`list` <:obj:`str`>) beamtime directories
         self.__beamtime_dirs = [
             # "/gpfs/current",
@@ -61,14 +65,33 @@ class BeamtimeWatcher:
         ]
         if "beamtime_dirs" in self.__config.keys() \
            and isinstance(self.__config["beamtime_dirs"], list):
-            self.__beamtime_dirs = self.__config["beamtime_dirs"]
+            self.__beamtime_dirs = []
+            for bdir in self.__config["beamtime_dirs"]:
+                if bdir:
+                    self.__beamtime_dirs.append(bdir.format(
+                        homepath=self.__homepath))
 
         #: (:obj:`str`) beamtime base directories
         self.__beamtime_base_dir = ""
         if "beamtime_base_dir" in self.__config.keys() \
            and self.__config["beamtime_base_dir"]:
             self.__beamtime_base_dir = os.path.abspath(
-                self.__config["beamtime_base_dir"])
+                self.__config["beamtime_base_dir"].format(
+                    homepath=self.__homepath))
+
+        #: (:obj:`list` <:obj:`str`>) scandir blacklist
+        self.__scandir_blacklist = [
+            "/gpfs/current/scratch_bl",
+            "/gpfs/current/processed",
+            "/gpfs/current/shared"
+        ]
+        if "scandir_blacklist" in self.__config.keys() \
+           and isinstance(self.__config["scandir_blacklist"], list):
+            self.__scandir_blacklist = []
+            for sdir in self.__config["scandir_blacklist"]:
+                if sdir:
+                    self.__scandir_blacklist.append(sdir.format(
+                        homepath=self.__homepath))
 
         #: (:obj:`bool`) access groups from proposals
         self.__groups_from_proposal = False
@@ -77,21 +100,30 @@ class BeamtimeWatcher:
                 self.__config["owner_access_groups_from_proposal"]
 
         #: (:obj:`dict` <:obj:`str`, :obj:`str`>)
-        #                             beamtime path to watcher path map
+        #:                            beamtime path to watcher path map
         self.__wait_for_dirs = {}
 
+        #: (:obj:`int`) maximal scandir depth
+        self.__scandir_depth = -1
+
+        if "max_scandir_depth" in self.__config.keys():
+            try:
+                self.__scandir_depth = int(self.__config["max_scandir_depth"])
+            except Exception as e:
+                get_logger().warning('%s' % (str(e)))
+
         #: (:obj:`int`) notifier id
         self.__notifier = None
         #: (:obj:`dict` <:obj:`int`, :obj:`str`>) watch description paths
         self.__wd_to_path = {}
         #: (:obj:`dict` <:obj:`int`, :obj:`str`>)
-        #                               beamtime watch description paths
+        #:                              beamtime watch description path queues
         self.__wd_to_queue = {}
         #: (:obj:`dict` <:obj:`int`, :class:`queue.Queue`>)
-        #                               beamtime watch description paths
+        #:                              beamtime watch description base paths
         self.__wd_to_bpath = {}
         #: (:obj:`dict` <:obj:`int`, :class:`queue.Queue`>)
-        #                               beamtime watch description paths
+        #:                       beamtime watch description base path queues
         self.__wd_to_bqueue = {}
 
         #: (:obj:`str`) beamtime file prefix
@@ -99,9 +131,18 @@ class BeamtimeWatcher:
         #: (:obj:`str`) beamtime file postfix
         self.__bt_postfix = ".json"
 
+        #: (:obj:`float`) max count of recheck the beamtime files
+        self.__recheck_btfile_interval = 1000
+        if "recheck_beamtime_file_interval" in self.__config.keys():
+            try:
+                self.__recheck_btfile_interval = int(
+                    self.__config["recheck_beamtime_file_interval"])
+            except Exception as e:
+                get_logger().warning('%s' % (str(e)))
+
         #: (:obj:`dict` <(:obj:`str`, :obj:`str`),
-        #                :class:`scanDirWatcher.ScanDirWatcher`>)
-        #        scandir watchers instances for given path and beamtime file
+        #:               :class:`scanDirWatcher.ScanDirWatcher`>)
+        #:        scandir watchers instances for given path and beamtime file
         self.__scandir_watchers = {}
         #: (:class:`threading.Lock`) scandir watcher dictionary lock
         self.__scandir_lock = threading.Lock()
@@ -279,6 +320,7 @@ class BeamtimeWatcher:
                 self._launch_scandir_watcher(path, files)
                 get_logger().debug('Files of %s: %s' % (path, files))
 
+            counter = 0
             while self.running:
                 get_logger().debug('Bt Tic')
                 if not self.__wd_to_queue and not self.__wd_to_bqueue:
@@ -427,6 +469,50 @@ class BeamtimeWatcher:
                                     get_logger().debug(
                                         'Files of %s: %s' % (dr, files))
 
+                if self.__recheck_btfile_interval > 0:
+                    if counter == self.__recheck_btfile_interval:
+                        # if inotify does not work
+                        counter = 0
+                        # get_logger().info(
+                        #   'DatasetWatcher: Re-check dataset list after %s s'
+                        #   % self.__recheck_btfile_interval)
+                        get_logger().debug(
+                            'BeamtimeWatcher: '
+                            'Re-check beamtime file after %s s'
+                            % self.__recheck_btfile_interval)
+                        dds = []
+                        for (ph, fl) in list(self.__scandir_watchers.keys()):
+                            if not os.path.isfile(fl):
+                                ds = self.__scandir_watchers.pop((ph, fl))
+                                ds.running = False
+                                dds.append(ds)
+                        while len(dds):
+                            ds = dds.pop()
+                            ds.running = False
+                            get_logger().debug("Joining ScanDirWatcher")
+                            ds.join()
+                            get_logger().debug("ScanDirWatcher Joined")
+                        get_logger().debug('add paths')
+
+                        for path in self.__beamtime_dirs:
+                            files = self._find_bt_files(
+                                path, self.__bt_prefix, self.__bt_postfix)
+                            self._launch_scandir_watcher(path, files)
+
+                        # try:
+                        #     self.__ingestor.check_list()
+                        # except Exception as e:
+                        #     get_logger().warning(str(e))
+                        #     continue
+                    elif self.__recheck_btfile_interval > counter:
+                        get_logger().debug(
+                            'BeamtimeWatcher: increase counter %s/%s ' %
+                            (counter, self.__recheck_btfile_interval))
+                        # GET_logger().info(
+                        #     'DatasetWatcher: increase counter %s/%s ' %
+                        #     (counter, self.__recheck_btfile_interval))
+                        counter += 1
+
                 get_logger().debug(
                     "Running: %s s" % (time.time() - self.__starttime))
                 if self.__runtime and \
@@ -444,6 +530,8 @@ class BeamtimeWatcher:
         :param path: beamtime files
         :type path: :obj:`list`<:obj:`str`>
         """
+        if path in self.__scandir_blacklist:
+            return
         for bt in files:
             ffn = os.path.abspath(os.path.join(path, bt))
             try:
@@ -462,7 +550,8 @@ class BeamtimeWatcher:
                             % (path, ffn))
                         btmd = self.__append_proposal_groups(btmd, path)
                         sdw = self.__scandir_watchers[(path, ffn)] =  \
-                            ScanDirWatcher(self.__config, path, btmd, ffn)
+                            ScanDirWatcher(self.__config, path, btmd, ffn,
+                                           self.__scandir_depth)
                 if sdw is not None:
                     sdw.start()
             except Exception as e:
@@ -538,6 +627,14 @@ def main():
         "-l", "--log", dest="log",
         help="logging level, i.e. debug, info, warning, error, critical",
         default="info")
+    parser.add_argument(
+        "-f", "--log-file", dest="logfile",
+        help="log file name")
+    parser.add_argument(
+        "-t", "--timestamps", action="store_true",
+        default=False, dest="timestamps",
+        help="timestamps in logs")
+
     try:
         options = parser.parse_args()
     except Exception as e:
@@ -547,7 +644,8 @@ def main():
         print("")
         sys.exit(255)
 
-    init_logger("SciCatDatasetIngestor", options.log)
+    init_logger("SciCatDatasetIngestor", options.log,
+                options.timestamps, options.logfile)
 
     bw = BeamtimeWatcher(options)
     bw.start()
