@@ -23,9 +23,12 @@ import argparse
 import os
 import glob
 import json
+import socket
+import pathlib
 
 from .configuration import load_config
 from .datasetIngestor import DatasetIngestor
+from .pathConverter import PathConverter
 from .logger import get_logger, init_logger
 
 
@@ -46,24 +49,52 @@ class DatasetIngest:
             self.__config = load_config(options.config) or {}
             get_logger().debug("CONFIGURATION: %s" % str(self.__config))
 
+        #: (:obj:`bool`) use core path
+        self.__usecorepath = False
+        if "use_corepath_as_scandir" in self.__config.keys():
+            try:
+                self.__usecorepath = bool(
+                    self.__config["use_corepath_as_scandir"])
+            except Exception as e:
+                get_logger().warning('%s' % (str(e)))
+
         #: (:obj:`list` <:obj:`str`>) beamtime directories
         self.__beamtime_dirs = [
             # "/gpfs/current",
             # "/gpfs/commissioning",
         ]
 
+        #: (:obj:`str`) home directory
+        self.__homepath = str(pathlib.Path.home())
+
         #: (:obj:`str`) beamtime file prefix
         self.__bt_prefix = "beamtime-metadata-"
         #: (:obj:`str`) beamtime file postfix
         self.__bt_postfix = ".json"
 
         #: (:obj:`str`) scicat dataset file pattern
-        self.__ds_pattern = "scicat-datasets-{bt}.lst"
+        self.__ds_pattern = "scicat-datasets-{beamtimeid}.lst"
         #: (:obj:`str`) indested scicat dataset file pattern
-        self.__ids_pattern = "scicat-ingested-datasets-{bt}.lst"
+        self.__ids_pattern = "scicat-ingested-datasets-{beamtimeid}.lst"
+        #: (:obj:`str`) indested scicat dataset file pattern
+        self.__hostname = socket.gethostname()
 
         #: (:obj:`str`) ingestor log directory
-        self.__log_dir = ""
+        self.__var_dir = ""
+
+        #: (:obj:`list` <:obj:`str`>) scandir blacklist
+        self.__scandir_blacklist = [
+            "/gpfs/current/scratch_bl",
+            "/gpfs/current/processed",
+            "/gpfs/current/shared"
+        ]
+        if "scandir_blacklist" in self.__config.keys() \
+           and isinstance(self.__config["scandir_blacklist"], list):
+            self.__scandir_blacklist = []
+            for sdir in self.__config["scandir_blacklist"]:
+                if sdir:
+                    self.__scandir_blacklist.append(sdir.format(
+                        homepath=self.__homepath))
 
         #: (:obj:`bool`) access groups from proposals
         self.__groups_from_proposal = False
@@ -73,12 +104,16 @@ class DatasetIngest:
 
         if "beamtime_dirs" in self.__config.keys() \
            and isinstance(self.__config["beamtime_dirs"], list):
-            self.__beamtime_dirs = self.__config["beamtime_dirs"]
+            self.__beamtime_dirs = []
+            for bdir in self.__config["beamtime_dirs"]:
+                if bdir:
+                    self.__beamtime_dirs.append(bdir.format(
+                        homepath=self.__homepath))
 
-        if "ingestor_log_dir" in self.__config.keys():
-            self.__log_dir = self.__config["ingestor_log_dir"]
-        if self.__log_dir == "/":
-            self.__log_dir = ""
+        if "ingestor_var_dir" in self.__config.keys():
+            self.__var_dir = self.__config["ingestor_var_dir"]
+        if self.__var_dir == "/":
+            self.__var_dir = ""
 
         if "beamtime_filename_prefix" in self.__config.keys():
             self.__bt_prefix = self.__config["beamtime_filename_prefix"]
@@ -122,43 +157,57 @@ class DatasetIngest:
                     get_logger().warning(
                         "%s cannot be ingested: %s" % (ffn, str(e)))
 
-    def _ingest_scandir(self, path, meta, bpath):
+    def _ingest_scandir(self, path, meta, beamtimefile):
         """ constructor
 
         :param path: scan dir path
         :type path: :obj:`str`
         :param meta: beamtime configuration
         :type meta: :obj:`dict` <:obj:`str`, `any`>
-        :param bpath: beamtime file
-        :type bpath: :obj:`str`
+        :param beamtimefile: beamtime file
+        :type beamtimefile: :obj:`str`
         """
-        # #: (:obj:`str`) scan dir path
-        # self.__path = path
-        # #: (:obj:`str`) beamtime path and file name
-        # self.__bpath = bpath
-        # #: (:obj:`dict` <:obj:`str`, `any`>) beamtime configuration
-        # self.__meta = meta
         #: (:obj:`str`) beamtime id
         beamtimeId = meta["beamtimeId"]
 
+        #: (:obj:`str`) core path
+        corepath = meta.get("corePath", None)
+
+        #: (:obj:`str`) beamtime path
+        bpath = os.path.split(beamtimefile)[0]
+
+        conv = PathConverter(
+            corepath, bpath,
+            self.__usecorepath and corepath)
+
+        path = conv.to_core(path)
+
         #: (:obj:`str`) datasets file name
-        dslist_filename = self.__ds_pattern.format(bt=beamtimeId)
+        dslist_filename = self.__ds_pattern.format(
+            beamtimeid=beamtimeId, hostname=self.__hostname)
         #: (:obj:`str`) ingescted datasets file name
-        idslist_filename = self.__ids_pattern.format(bt=beamtimeId)
+        idslist_filename = self.__ids_pattern.format(
+            beamtimeid=beamtimeId, hostname=self.__hostname)
         dslfiles = glob.glob(
             "%s/**/%s" % (path, dslist_filename), recursive=True)
         for fn in dslfiles:
             get_logger().info("DatasetIngest: dataset list: %s" % str(fn))
             ifn = fn[:-(len(dslist_filename))] + idslist_filename
-            if self.__log_dir:
-                ifn = "%s%s" % (self.__log_dir, ifn)
+            if self.__var_dir:
+                ifn = "%s%s" % (
+                    self.__var_dir.format(
+                        beamtimeid=beamtimeId,
+                        homepath=self.__homepath), ifn)
             ipath, _ = os.path.split(ifn)
             if not os.path.isdir(ipath):
                 os.makedirs(ipath, exist_ok=True)
             scpath, pfn = os.path.split(fn)
+            if conv.to_core(scpath) in self.__scandir_blacklist or \
+               conv.from_core(scpath) in self.__scandir_blacklist:
+                continue
             ingestor = DatasetIngestor(
                 self.__config,
-                scpath, fn, ifn, meta, bpath)
+                scpath, fn, ifn, meta, conv.to_core(beamtimefile))
             if self.__groups_from_proposal and \
                ("accessGroups" not in meta or "ownerGroup" not in meta):
                 meta = ingestor.append_proposal_groups()
@@ -217,6 +266,14 @@ def main():
         "-l", "--log", dest="log",
         help="logging level, i.e. debug, info, warning, error, critical",
         default="info")
+    parser.add_argument(
+        "-f", "--log-file", dest="logfile",
+        help="log file name")
+    parser.add_argument(
+        "-t", "--timestamps", action="store_true",
+        default=False, dest="timestamps",
+        help="timestamps in logs")
+
     try:
         options = parser.parse_args()
     except Exception as e:
@@ -226,7 +283,8 @@ def main():
         print("")
         sys.exit(255)
 
-    init_logger("SciCatDatasetIngestor", options.log)
+    init_logger("SciCatDatasetIngest", options.log,
+                options.timestamps, options.logfile)
 
     di = DatasetIngest(options)
     di.start()
