@@ -24,6 +24,8 @@ import subprocess
 import requests
 import time
 import enum
+import socket
+import pathlib
 
 from .logger import get_logger
 
@@ -32,9 +34,16 @@ class UpdateStrategy(enum.Enum):
 
     """ Update strategy
     """
-    PATCH = 0
-    CREATE = 1
-    MIXED = 2
+    #: (:class:`scingestor.datasetIngestor.UpdateStrategy`)
+    #:       leave datasets unchanged
+    NO = 0
+    #: (:class:`scingestor.datasetIngestor.UpdateStrategy`) patch datasets
+    PATCH = 1
+    #: (:class:`scingestor.datasetIngestor.UpdateStrategy`) recreate datasets
+    CREATE = 2
+    #: (:class:`scingestor.datasetIngestor.UpdateStrategy`) patch datasets only
+    #:       if scientificMetadata changed otherwise recreate datasets
+    MIXED = 3
 
 
 class DatasetIngestor:
@@ -58,8 +67,8 @@ class DatasetIngestor:
         :type meta: :obj:`dict` <:obj:`str`, `any`>
         :param beamtimefile: beamtime filename
         :type beamtimefile: :obj:`str`
-        :param doiprefix: doiprefix
-        :type doiprefix: :obj:`str`
+        :param pidprefix: pidprefix
+        :type pidprefix: :obj:`str`
         :param ingestorcred: ingestor credential
         :type ingestorcred: :obj:`str`
         :param scicat_url: scicat_url
@@ -67,6 +76,8 @@ class DatasetIngestor:
         """
         #: (:obj:`dict` <:obj:`str`, `any`>) ingestor configuration
         self.__config = configuration or {}
+        #: (:obj:`str`) home directory
+        self.__homepath = str(pathlib.Path.home())
         #: (:obj:`str`) file with a dataset list
         self.__dsfile = dsfile
         #: (:obj:`str`) file with a ingested dataset list
@@ -85,13 +96,16 @@ class DatasetIngestor:
         self.__bfile = beamtimefile
         #: (:obj:`dict` <:obj:`str`, `any`>) beamtime metadata
         self.__meta = meta
+        #: (:obj:`str`) indested scicat dataset file pattern
+        self.__hostname = socket.gethostname()
 
         bpath, _ = os.path.split(beamtimefile)
         #: (:obj:`str`) relative scan path to beamtime path
         self.__relpath = os.path.relpath(path, bpath)
 
         #: (:obj:`str`) doi prefix
-        self.__doiprefix = "10.3204"
+        self.__pidprefix = ""
+        # self.__pidprefix = "10.3204"
         #: (:obj:`str`) username
         self.__username = 'ingestor'
         #: (:obj:`str`) update strategy
@@ -112,8 +126,12 @@ class DatasetIngestor:
         self.__scicat_datablocks = "OrigDatablocks"
         #: (:obj:`str`) chmod string for json metadata
         self.__chmod = None
+        #: (:obj:`str`) hidden attributes
+        self.__hiddenattributes = None
         #: (:obj:`bool`) oned metadata flag
         self.__oned = False
+        #: (:obj:`bool`) empty units flag
+        self.__emptyunits = True
 
         #: (:obj:`int`) maximal counter value for post tries
         self.__maxcounter = 100
@@ -125,33 +143,44 @@ class DatasetIngestor:
 
         #: (:obj:`str`) nexus dataset shell command
         self.__datasetcommandnxs = "nxsfileinfo metadata " \
-            " -o {metapath}/{scanname}{scpostfix} " \
+            " -o {metapath}/{scanname}{scanpostfix} " \
             " -b {beamtimefile} -p {beamtimeid}/{scanname} " \
             " -w {ownergroup}" \
             " -c {accessgroups}" \
             " {scanpath}/{scanname}.nxs"
         #: (:obj:`str`) datablock shell command
         self.__datasetcommand = "nxsfileinfo metadata " \
-            " -o {metapath}/{scanname}{scpostfix} " \
+            " -o {metapath}/{scanname}{scanpostfix} " \
             " -c {accessgroups}" \
             " -w {ownergroup}" \
             " -b {beamtimefile} -p {beamtimeid}/{scanname}"
         #: (:obj:`str`) datablock shell command
         self.__datablockcommand = "nxsfileinfo origdatablock " \
-            " -s *.pyc,*{dbpostfix},*{scpostfix},*~ " \
-            " -p {doiprefix}/{beamtimeid}/{scanname} " \
+            " -s *.pyc,*{datablockpostfix},*{scanpostfix},*~ " \
+            " -p {pidprefix}/{beamtimeid}/{scanname} " \
             " -w {ownergroup}" \
             " -c {accessgroups}" \
-            " -o {metapath}/{scanname}{dbpostfix} "
+            " -o {metapath}/{scanname}{datablockpostfix} "
         #: (:obj:`str`) datablock shell command
         self.__datablockmemcommand = "nxsfileinfo origdatablock " \
-            " -s *.pyc,*{dbpostfix},*{scpostfix},*~ " \
+            " -s *.pyc,*{datablockpostfix},*{scanpostfix},*~ " \
             " -w {ownergroup}" \
             " -c {accessgroups}" \
-            " -p {doiprefix}/{beamtimeid}/{scanname} "
+            " -p {pidprefix}/{beamtimeid}/{scanname} "
         #: (:obj:`str`) datablock path postfix
         self.__datablockscanpath = " {scanpath}/{scanname} "
 
+        #: (:obj:`str`) oned generator switch
+        self.__oned_switch = " --oned  "
+        #: (:obj:`str`) empty units generator switch
+        self.__emptyunits_switch = " --add-empty-units  "
+        #: (:obj:`str`) chmod generator switch
+        self.__chmod_switch = " -x {chmod} "
+        #: (:obj:`str`) hidden attributes generator switch
+        self.__hiddenattributes_switch = " -n {hiddenattributes} "
+        #: (:obj:`str`) relpath generator switch
+        self.__relpath_switch = " -r {relpath} "
+
         #: (:obj:`dict` <:obj:`str`, :obj:`str`>) request headers
         self.__headers = {'Content-Type': 'application/json',
                           'Accept': 'application/json'}
@@ -179,50 +208,55 @@ class DatasetIngestor:
         #: (:obj:`list`<:obj:`str`>) waiting scan names
         self.__sc_waiting = []
         #: (:obj:`dict`<:obj:`str`, :obj:`list`<:obj:`str`>>)
-        #   ingested scan names
+        #:   ingested scan names
         self.__sc_ingested_map = {}
 
         #: (:obj:`str`) access groups
         self.__accessgroups = \
-            "{beamtimeid}-clbt,{beamtimeid}-dmgt,{beamline}dmgt".format(
+            "{beamtimeid}-dmgt,{beamtimeid}-clbt,{beamtimeid}-part," \
+            "{beamline}dmgt,{beamline}staff".format(
                 beamtimeid=self.__bid, beamline=self.__bl)
         if "accessGroups" in self.__meta:
             self.__accessgroups = ",".join(self.__meta["accessGroups"])
 
         #: (:obj:`str`) owner group
         self.__ownergroup = \
-            "{beamtimeid}-part".format(
+            "{beamtimeid}-dmgt".format(
                 beamtimeid=self.__bid)
         if "ownerGroup" in self.__meta:
             self.__ownergroup = self.__meta["ownerGroup"]
 
         #: (:obj:`bool`) metadata in log dir flag
-        self.__meta_in_log_dir = False
-        if "metadata_in_log_dir" in self.__config.keys():
-            self.__meta_in_log_dir = self.__config["metadata_in_log_dir"]
+        self.__meta_in_var_dir = False
+        if "metadata_in_var_dir" in self.__config.keys():
+            self.__meta_in_var_dir = self.__config["metadata_in_var_dir"]
 
         #: (:obj:`str`) ingestor log directory
-        self.__log_dir = ""
-        if "ingestor_log_dir" in self.__config.keys():
-            self.__log_dir = self.__config["ingestor_log_dir"]
-        if self.__log_dir == "/":
-            self.__log_dir = ""
-        if self.__meta_in_log_dir and self.__log_dir:
-            self.__metapath = "%s%s" % (self.__log_dir, self.__metapath)
+        self.__var_dir = ""
+        if "ingestor_var_dir" in self.__config.keys():
+            self.__var_dir = str(
+                self.__config["ingestor_var_dir"]).format(
+                    beamtimeid=self.__bid,
+                    homepath=self.__homepath)
+        if self.__var_dir == "/":
+            self.__var_dir = ""
+        if self.__meta_in_var_dir and self.__var_dir:
+            self.__metapath = "%s%s" % (self.__var_dir, self.__metapath)
             if not os.path.isdir(self.__metapath):
                 os.makedirs(self.__metapath, exist_ok=True)
 
-        if "doiprefix" in self.__config.keys():
-            self.__doiprefix = self.__config["doi_prefix"]
+        if "dataset_pid_prefix" in self.__config.keys():
+            self.__pidprefix = self.__config["dataset_pid_prefix"]
         if "ingestor_credential_file" in self.__config.keys():
-            with open(self.__config["ingestor_credential_file"]) as fl:
+            with open(self.__config["ingestor_credential_file"].format(
+                    homepath=self.__homepath)) as fl:
                 self.__incd = fl.read().strip()
         if "ingestor_username" in self.__config.keys():
             self.__username = self.__config["ingestor_username"]
-        if "update_strategy" in self.__config.keys():
+        if "dataset_update_strategy" in self.__config.keys():
             try:
                 self.__strategy = UpdateStrategy[
-                    str(self.__config["update_strategy"]).upper()]
+                    str(self.__config["dataset_update_strategy"]).upper()]
             except Exception as e:
                 get_logger().warning(
                     'Wrong UpdateStrategy value: %s' % str(e))
@@ -244,8 +278,12 @@ class DatasetIngestor:
                 self.__config["relative_path_in_datablock"]
         if "chmod_json_files" in self.__config.keys():
             self.__chmod = self.__config["chmod_json_files"]
+        if "hidden_attributes" in self.__config.keys():
+            self.__hiddenattributes = self.__config["hidden_attributes"]
         if "oned_in_metadata" in self.__config.keys():
             self.__oned = self.__config["oned_in_metadata"]
+        if "add_empty_units" in self.__config.keys():
+            self.__emptyunits = self.__config["add_empty_units"]
 
         if "scan_metadata_postfix" in self.__config.keys():
             self.__scanpostfix = self.__config["scan_metadata_postfix"]
@@ -270,44 +308,79 @@ class DatasetIngestor:
             self.__datablockscanpath = \
                 self.__config["datablock_metadata_generator_scanpath_postfix"]
 
+        if "chmod_generator_switch" in self.__config.keys():
+            self.__chmod_switch = \
+                self.__config["chmod_generator_switch"]
+
+        if "hidden_attributes_generator_switch" in self.__config.keys():
+            self.__hiddenattributes_switch = \
+                self.__config["hidden_attributes_generator_switch"]
+
+        if "relative_path_generator_switch" in self.__config.keys():
+            self.__relpath_switch = \
+                self.__config["relative_path_generator_switch"]
+
+        if "oned_dataset_generator_switch" in self.__config.keys():
+            self.__oned_switch = \
+                self.__config["oned_dataset_generator_switch"]
+
+        if "add_empty_units_generator_switch" in self.__config.keys():
+            self.__emptyunits_switch = \
+                self.__config["add_empty_units_generator_switch"]
+
         if self.__relpath_in_datablock:
             if "datablock_metadata_generator" not in self.__config.keys():
                 self.__datablockcommand = \
-                    self.__datablockcommand + " -r {relpath} "
+                    self.__datablockcommand + self.__relpath_switch
             if "datablock_metadata_stream_generator" \
                not in self.__config.keys():
                 self.__datablockmemcommand = \
-                    self.__datablockmemcommand + " -r {relpath} "
+                    self.__datablockmemcommand + self.__relpath_switch
         else:
             if "dataset_metadata_generator" not in self.__config.keys():
                 self.__datasetcommand = \
-                    self.__datasetcommand + " -r {relpath} "
+                    self.__datasetcommand + self.__relpath_switch
             if "nxs_dataset_metadata_generator" not in self.__config.keys():
                 self.__datasetcommandnxs = \
-                    self.__datasetcommandnxs + " -r {relpath} "
+                    self.__datasetcommandnxs + self.__relpath_switch
 
         if self.__chmod is not None:
             if "dataset_metadata_generator" not in self.__config.keys():
                 self.__datasetcommand = \
-                    self.__datasetcommand + " -x {chmod} "
+                    self.__datasetcommand + self.__chmod_switch
             if "nxs_dataset_metadata_generator" not in self.__config.keys():
                 self.__datasetcommandnxs = \
-                    self.__datasetcommandnxs + " -x {chmod} "
+                    self.__datasetcommandnxs + self.__chmod_switch
             if "datablock_metadata_generator" not in self.__config.keys():
                 self.__datablockcommand = \
-                    self.__datablockcommand + " -x {chmod} "
+                    self.__datablockcommand + self.__chmod_switch
             if "datablock_metadata_stream_generator" \
                not in self.__config.keys():
                 self.__datablockmemcommand = \
-                    self.__datablockmemcommand + " -x {chmod} "
+                    self.__datablockmemcommand + self.__chmod_switch
 
+        if self.__hiddenattributes is not None:
+            if "dataset_metadata_generator" not in self.__config.keys():
+                self.__datasetcommand = \
+                    self.__datasetcommand + self.__hiddenattributes_switch
+            if "nxs_dataset_metadata_generator" not in self.__config.keys():
+                self.__datasetcommandnxs = \
+                    self.__datasetcommandnxs + self.__hiddenattributes_switch
         if self.__oned:
             if "dataset_metadata_generator" not in self.__config.keys():
                 self.__datasetcommand = \
-                    self.__datasetcommand + " --oned "
+                    self.__datasetcommand + self.__oned_switch
             if "nxs_dataset_metadata_generator" not in self.__config.keys():
                 self.__datasetcommandnxs = \
-                    self.__datasetcommandnxs + " --oned "
+                    self.__datasetcommandnxs + self.__oned_switch
+
+        if self.__emptyunits:
+            if "dataset_metadata_generator" not in self.__config.keys():
+                self.__datasetcommand = \
+                    self.__datasetcommand + self.__emptyunits_switch
+            if "nxs_dataset_metadata_generator" not in self.__config.keys():
+                self.__datasetcommandnxs = \
+                    self.__datasetcommandnxs + self.__emptyunits_switch
 
         if "max_request_tries_number" in self.__config.keys():
             try:
@@ -334,17 +407,20 @@ class DatasetIngestor:
         self.__dctfmt = {
             "scanname": None,
             "chmod": self.__chmod,
+            "hiddenattributes": self.__hiddenattributes,
             "scanpath": self.__path,
             "metapath": self.__metapath,
             "relpath": self.__relpath,
             "beamtimeid": self.__bid,
             "beamline": self.__bl,
-            "doiprefix": self.__doiprefix,
+            "pidprefix": self.__pidprefix,
             "beamtimefile": self.__bfile,
-            "scpostfix": self.__scanpostfix,
-            "dbpostfix": self.__datablockpostfix,
+            "scanpostfix": self.__scanpostfix,
+            "datablockpostfix": self.__datablockpostfix,
             "ownergroup": self.__ownergroup,
-            "accessgroups": self.__accessgroups
+            "accessgroups": self.__accessgroups,
+            "hostname": self.__hostname,
+            "homepath": self.__homepath
         }
 
         get_logger().debug(
@@ -386,7 +462,7 @@ class DatasetIngestor:
             get_logger().info(
                 'DatasetIngestor: Generating nxs metadata: %s %s' % (
                     scan,
-                    "{metapath}/{scanname}{scpostfix}".format(
+                    "{metapath}/{scanname}{scanpostfix}".format(
                         **self.__dctfmt)))
             get_logger().debug(
                 'DatasetIngestor: Generating dataset command: %s ' % (
@@ -398,7 +474,7 @@ class DatasetIngestor:
             get_logger().info(
                 'DatasetIngestor: Generating metadata: %s %s' % (
                     scan,
-                    "{metapath}/{scanname}{scpostfix}".format(
+                    "{metapath}/{scanname}{scanpostfix}".format(
                         **self.__dctfmt)))
             get_logger().debug(
                 'DatasetIngestor: Generating dataset command: %s ' % (
@@ -408,10 +484,11 @@ class DatasetIngestor:
                 check=True)
 
         rdss = glob.glob(
-            "{metapath}/{scanname}{scpostfix}".format(
+            "{metapath}/{scanname}{scanpostfix}".format(
                         **self.__dctfmt))
         if rdss and rdss[0]:
             return rdss[0]
+
         return ""
 
     def _generate_origdatablock_metadata(self, scan):
@@ -425,7 +502,7 @@ class DatasetIngestor:
         get_logger().info(
             'DatasetIngestor: Generating origdatablock metadata: %s %s' % (
                 scan,
-                "{metapath}/{scanname}{dbpostfix}".format(
+                "{metapath}/{scanname}{datablockpostfix}".format(
                     **self.__dctfmt)))
         cmd = self.__datablockcommand.format(**self.__dctfmt)
         sscan = (scan or "").split(" ")
@@ -438,7 +515,7 @@ class DatasetIngestor:
         #     'DatasetIngestor: Generating origdatablock command: %s ' % cmd)
         subprocess.run(cmd.split(), check=True)
         odbs = glob.glob(
-            "{metapath}/{scanname}{dbpostfix}".format(
+            "{metapath}/{scanname}{datablockpostfix}".format(
                     **self.__dctfmt))
         if odbs and odbs[0]:
             return odbs[0]
@@ -454,7 +531,7 @@ class DatasetIngestor:
         :returns: a file name of generate file
         :rtype: :obj:`str`
         """
-        mfilename = "{metapath}/{scanname}{dbpostfix}".format(
+        mfilename = "{metapath}/{scanname}{datablockpostfix}".format(
             **self.__dctfmt)
         get_logger().info(
             'DatasetIngestor: Checking origdatablock metadata: %s %s' % (
@@ -503,13 +580,13 @@ class DatasetIngestor:
                         'DatasetIngestor: '
                         'Generating origdatablock metadata: %s %s' % (
                             scan,
-                            "{metapath}/{scanname}{dbpostfix}".format(
+                            "{metapath}/{scanname}{datablockpostfix}".format(
                                 **self.__dctfmt)))
                     with open(mfilename, "w") as mf:
                         mf.write(nwmeta)
 
         odbs = glob.glob(
-            "{metapath}/{scanname}{dbpostfix}".format(
+            "{metapath}/{scanname}{datablockpostfix}".format(
                     **self.__dctfmt))
         if odbs and odbs[0]:
             return odbs[0]
@@ -749,7 +826,7 @@ class DatasetIngestor:
         :rtype: :obj:`str`
         """
         try:
-            pid = "%s/%s" % (self.__doiprefix, mdct["pid"])
+            pid = "%s/%s" % (self.__pidprefix, mdct["pid"])
             # check if dataset with the pid exists
             get_logger().info(
                 'DatasetIngestor: Check if dataset exists: %s' % (pid))
@@ -789,7 +866,7 @@ class DatasetIngestor:
                         return mdct["pid"]
                     else:
                         raise Exception("%s" % response.text)
-                else:
+                elif self.__strategy != UpdateStrategy.NO:
                     # find dataset by pid
                     get_logger().info(
                         'DatasetIngestor: Find the dataset by id: %s' % (pid))
@@ -804,7 +881,8 @@ class DatasetIngestor:
                         mdic["pid"] = pid
                         if not self._metadataEqual(
                                 dsmeta, mdic, skip=self.__withoutsm):
-                            if self.__strategy == UpdateStrategy.PATCH:
+                            if self.__strategy in [
+                                    UpdateStrategy.PATCH, UpdateStrategy.NO]:
                                 nmeta = json.dumps(mdic)
                                 return self._patch_dataset(
                                     nmeta, pid, token, mdct)
@@ -826,6 +904,8 @@ class DatasetIngestor:
                                             nmeta, pid, token, mdct)
                     else:
                         raise Exception("%s" % resds.text)
+                else:
+                    return pid
             else:
                 raise Exception("%s" % resexists.text)
         except Exception as e:
@@ -911,12 +991,12 @@ class DatasetIngestor:
                 smt = fl.read()
                 mt = json.loads(smt)
             if not mt["datasetId"].startswith(
-                    "%s/%s/" % (self.__doiprefix, self.__bid)):
+                    "%s/%s/" % (self.__pidprefix, self.__bid)):
                 raise Exception(
                     "Wrong datasetId %s for DESY beamtimeId %s in  %s"
                     % (mt["pid"], self.__bid, metafile))
-            if mt["datasetId"] != "%s/%s" % (self.__doiprefix, pid):
-                mt["datasetId"] = "%s/%s" % (self.__doiprefix, pid)
+            if mt["datasetId"] != "%s/%s" % (self.__pidprefix, pid):
+                mt["datasetId"] = "%s/%s" % (self.__pidprefix, pid)
                 smt = json.dumps(mt)
                 with open(metafile, "w") as mf:
                     mf.write(smt)
@@ -942,7 +1022,6 @@ class DatasetIngestor:
 
         sscan = scan.split(" ")
         self.__dctfmt["scanname"] = sscan[0] if len(sscan) > 0 else ""
-
         rdss = glob.glob(
             "{metapath}/{scan}{postfix}".format(
                 scan=self.__dctfmt["scanname"],
@@ -1005,6 +1084,7 @@ class DatasetIngestor:
         reingest_origdatablock = False
         sscan = scan.split(" ")
         self.__dctfmt["scanname"] = sscan[0] if len(sscan) > 0 else ""
+        rds = None
         rdss = glob.glob(
             "{metapath}/{scan}{postfix}".format(
                 scan=self.__dctfmt["scanname"],
@@ -1023,9 +1103,11 @@ class DatasetIngestor:
                     mtm > self.__sc_ingested_map[scan][-2]))
             if scan not in self.__sc_ingested_map.keys() \
                or mtm > self.__sc_ingested_map[scan][-2]:
-                reingest_dataset = True
+                if self.__strategy != UpdateStrategy.NO:
+                    reingest_dataset = True
         else:
-            rds = self._generate_rawdataset_metadata(self.__dctfmt["scanname"])
+            rds = self._generate_rawdataset_metadata(
+                self.__dctfmt["scanname"])
             get_logger().debug("DS No File: %s True" % (scan))
             reingest_dataset = True
         mtmds = 0
