@@ -19,10 +19,12 @@ import os
 import threading
 import time
 import queue
-
+import socket
+import pathlib
 
 from .datasetWatcher import DatasetWatcher
 from .safeINotifier import SafeINotifier
+from .pathConverter import PathConverter
 from .logger import get_logger
 
 import inotifyx
@@ -33,7 +35,7 @@ class ScanDirWatcher(threading.Thread):
     """
     def __init__(self,
                  configuration,
-                 path, meta, bpath):
+                 path, meta, beamtimefile, depth):
         """ constructor
 
         :param configuration: dictionary with the ingestor configuration
@@ -42,8 +44,10 @@ class ScanDirWatcher(threading.Thread):
         :type path: :obj:`str`
         :param meta: beamtime configuration
         :type meta: :obj:`dict` <:obj:`str`, `any`>
-        :param bpath: beamtime file
-        :type bpath: :obj:`str`
+        :param beamtimefile: beamtime file
+        :type beamtimefile: :obj:`str`
+        :param depth: scandir depth level
+        :type depth: :obj:`int`
         """
         threading.Thread.__init__(self)
 
@@ -52,32 +56,58 @@ class ScanDirWatcher(threading.Thread):
 
         #: (:obj:`dict` <:obj:`str`, `any`>) ingestor configuration
         self.__config = configuration or {}
+
+        #: (:obj:`str`) home directory
+        self.__homepath = str(pathlib.Path.home())
+
+        #: (:obj:`str`) core path
+        self.__corepath = meta.get("corePath", None)
+
+        #: (:obj:`str`) beamtime path
+        self.__bpath = os.path.split(beamtimefile)[0]
+
+        #: (:obj:`bool`) use core path
+        self.__usecorepath = False
+        if "use_corepath_as_scandir" in self.__config.keys():
+            try:
+                self.__usecorepath = bool(
+                    self.__config["use_corepath_as_scandir"])
+            except Exception as e:
+                get_logger().warning('%s' % (str(e)))
+
+        #: (:class:`scingestor.datasetWatcher.DatasetWatcher`) use core path
+        self.__conv = PathConverter(
+            self.__corepath, self.__bpath,
+            self.__usecorepath and self.__corepath)
         #: (:obj:`str`) scan dir path
-        self.__path = path
-        #: (:obj:`str`) beamtime path and file name
-        self.__bpath = bpath
+        self.__path = self.__conv.to_core(path)
+        #: (:obj:`str`) beamtime core path and file name
+        self.__btfile = self.__conv.to_core(beamtimefile)
         #: (:obj:`dict` <:obj:`str`, `any`>) beamtime configuration
         self.__meta = meta
+        #: (:obj:`int`) scan dir depth
+        self.__depth = depth
         #: (:obj:`str`) beamtime id
         self.__beamtimeId = meta["beamtimeId"]
-        #: (:obj:`str`) beamline metadata
-        self.__meta = meta
+
         #: (:obj:`str`) scicat dataset file pattern
-        self.__ds_pattern = "scicat-datasets-{bt}.lst"
+        self.__ds_pattern = "scicat-datasets-{beamtimeid}.lst"
         #: (:obj:`str`) indested scicat dataset file pattern
-        self.__ids_pattern = "scicat-ingested-datasets-{bt}.lst"
+        self.__ids_pattern = "scicat-ingested-datasets-{beamtimeid}.lst"
+        #: (:obj:`str`) indested scicat dataset file pattern
+        self.__hostname = socket.gethostname()
 
         #: (:obj:`int`) notifier ID
         self.__notifier = None
-        #: (:obj:`dict` <:obj:`int`, :obj:`str`>) watch description paths
+        #: (obj:`dict` <:obj:`int`, :obj:`str`>) watch description paths
         self.__wd_to_path = {}
         #: (:obj:`dict` <:obj:`int`, :obj:`str`>)
-        #                               beamtime watch description paths
+        #:                              beamtime watch description paths
         self.__wd_to_queue = {}
 
         #: (:obj:`dict` <(:obj:`str`, :obj:`str`),
-        #                :class:`scanDirWatcher.ScanDirWatcher`>)
-        #        dataset watchers instances for given path and beamtime file
+        #:               :class:`scanDirWatcher.ScanDirWatcher`>)
+        #:       dataset watchers instances for given path and beamtime file
         self.__dataset_watchers = {}
         #: (:class:`threading.Lock`) dataset watcher dictionary lock
         self.__dataset_lock = threading.Lock()
@@ -85,12 +115,26 @@ class ScanDirWatcher(threading.Thread):
         self.__timeout = 0.01
 
         #: (:obj:`dict` <(:obj:`str`, :obj:`str`),
-        #                :class:`scanDirWatcher.ScanDirWatcher`>)
-        #        scandir watchers instances for given path and beamtime file
+        #:               :class:`scanDirWatcher.ScanDirWatcher`>)
+        #:       scandir watchers instances for given path and beamtime file
         self.__scandir_watchers = {}
         #: (:class:`threading.Lock`) scandir watcher dictionary lock
         self.__scandir_lock = threading.Lock()
 
+        #: (:obj:`list` <:obj:`str`>) scandir blacklist
+        self.__scandir_blacklist = [
+            "/gpfs/current/scratch_bl",
+            "/gpfs/current/processed",
+            "/gpfs/current/shared"
+        ]
+        if "scandir_blacklist" in self.__config.keys() \
+           and isinstance(self.__config["scandir_blacklist"], list):
+            self.__scandir_blacklist = []
+            for sdir in self.__config["scandir_blacklist"]:
+                if sdir:
+                    self.__scandir_blacklist.append(sdir.format(
+                        homepath=self.__homepath))
+
         if "get_event_timeout" in self.__config.keys():
             try:
                 self.__timeout = float(self.__config["get_event_timeout"])
@@ -106,25 +150,28 @@ class ScanDirWatcher(threading.Thread):
 
         #: (:obj:`str`) datasets file name
         self.__dslist_filename = self.__ds_pattern.format(
-            bt=self.__beamtimeId)
+            beamtimeid=self.__beamtimeId, hostname=self.__hostname)
         #: (:obj:`str`) ingescted datasets file name
         self.__idslist_filename = self.__ids_pattern.format(
-            bt=self.__beamtimeId)
+            beamtimeid=self.__beamtimeId, hostname=self.__hostname)
         #: (:obj:`str`) datasets file name
         self.__dslist_fullname = os.path.join(
             self.__path, self.__dslist_filename)
 
         #: (:obj:`str`) ingestor log directory
-        self.__log_dir = ""
-        if "ingestor_log_dir" in self.__config.keys():
-            self.__log_dir = self.__config["ingestor_log_dir"]
-        if self.__log_dir == "/":
-            self.__log_dir = ""
+        self.__var_dir = ""
+        if "ingestor_var_dir" in self.__config.keys():
+            self.__var_dir = str(
+                self.__config["ingestor_var_dir"]).format(
+                    beamtimeid=self.__beamtimeId,
+                    homepath=self.__homepath)
+        if self.__var_dir == "/":
+            self.__var_dir = ""
 
     def _start_notifier(self, path):
         """ start notifier
 
-        :param path: beamtime file subpath
+        :param path: beamtime file subdirectory
         :type path: :obj:`str`
         """
         self.__notifier = SafeINotifier()
@@ -138,7 +185,7 @@ class ScanDirWatcher(threading.Thread):
         """
         try:
             wqueue, watch_descriptor = self.__notifier.add_watch(
-                path,
+                self.__conv.from_core(path),
                 inotifyx.IN_ALL_EVENTS |
                 inotifyx.IN_CLOSE_WRITE | inotifyx.IN_DELETE |
                 inotifyx.IN_MOVE_SELF |
@@ -169,25 +216,33 @@ class ScanDirWatcher(threading.Thread):
         :param path: list of subdirectories
         :type path: :obj:`list`<:obj:`str`>
         """
-        for path in sorted(paths):
-            sdw = None
-            try:
-                with self.__scandir_lock:
-                    if (path, self.__bpath) \
-                       not in self.__scandir_watchers.keys():
-                        sdw = \
-                            self.__scandir_watchers[(path, self.__bpath)] =  \
-                            ScanDirWatcher(self.__config,
-                                           path, self.__meta, self.__bpath)
-                        get_logger().info(
-                            'ScanDirWatcher: Create ScanDirWatcher %s %s'
-                            % (path, self.__bpath))
-                if sdw is not None:
-                    sdw.start()
-                time.sleep(self.__timeout/10.)
-            except Exception as e:
-                get_logger().warning(
-                    "%s cannot be watched: %s" % (path, str(e)))
+        if self.__depth != 0:
+            for path in sorted(paths):
+                if self.__conv.from_core(path) in self.__scandir_blacklist or \
+                   self.__conv.to_core(path) in self.__scandir_blacklist:
+                    continue
+                sdw = None
+                try:
+                    with self.__scandir_lock:
+                        if (path, self.__btfile) \
+                           not in self.__scandir_watchers.keys():
+                            sdw = \
+                                self.__scandir_watchers[
+                                    (path, self.__btfile)] = ScanDirWatcher(
+                                        self.__config,
+                                        self.__conv.from_core(path),
+                                        self.__meta,
+                                        self.__conv.from_core(self.__btfile),
+                                        self.__depth - 1)
+                            get_logger().info(
+                                'ScanDirWatcher: Create ScanDirWatcher %s %s'
+                                % (path, self.__btfile))
+                    if sdw is not None:
+                        sdw.start()
+                    time.sleep(self.__timeout/10.)
+                except Exception as e:
+                    get_logger().warning(
+                        "%s cannot be watched: %s" % (path, str(e)))
 
     def run(self):
         """ scandir watcher thread
@@ -204,14 +259,16 @@ class ScanDirWatcher(threading.Thread):
                     if fn not in self.__dataset_watchers.keys():
                         ifn = fn[:-(len(self.__dslist_filename))] + \
                             self.__idslist_filename
-                        if self.__log_dir:
-                            ifn = "%s%s" % (self.__log_dir, ifn)
+                        if self.__var_dir:
+                            ifn = "%s%s" % (self.__var_dir, ifn)
                         ipath, _ = os.path.split(ifn)
                         if not os.path.isdir(ipath):
                             os.makedirs(ipath, exist_ok=True)
                         dw = self.__dataset_watchers[fn] = DatasetWatcher(
                             self.__config,
-                            self.__path, fn, ifn, self.__meta, self.__bpath)
+                            self.__path,
+                            fn, ifn, self.__meta,
+                            self.__conv.from_core(self.__btfile))
                         get_logger().info(
                             'ScanDirWatcher: Creating DatasetWatcher %s' % fn)
                 if dw is not None:
@@ -275,8 +332,8 @@ class ScanDirWatcher(threading.Thread):
                                     ifn = \
                                         fn[:-(len(self.__dslist_filename))] \
                                         + self.__idslist_filename
-                                    if self.__log_dir:
-                                        ifn = "%s%s" % (self.__log_dir, ifn)
+                                    if self.__var_dir:
+                                        ifn = "%s%s" % (self.__var_dir, ifn)
                                     ipath, _ = os.path.split(ifn)
                                     if not os.path.isdir(ipath):
                                         os.makedirs(ipath, exist_ok=True)
@@ -284,7 +341,7 @@ class ScanDirWatcher(threading.Thread):
                                         DatasetWatcher(
                                             self.__config, self.__path,
                                             fn, ifn,
-                                            self.__meta, self.__bpath)
+                                            self.__meta, self.__btfile)
                             if dw is not None:
                                 dw.start()
                                 get_logger().info(
